package com.bomostory.sceneeditmodule

import android.media.AudioFormat
import android.media.AudioRecord
import android.media.MediaRecorder
import android.os.Build
import com.naman14.androidlame.AndroidLame
import com.naman14.androidlame.LameBuilder
import java.io.File
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.IOException
import java.util.concurrent.locks.ReentrantLock


class AudioRecorder {

    private var sampleRate = 44100
    private var recorderChannelConfig = AudioFormat.CHANNEL_IN_MONO
    private var recorderAudioFormat = AudioFormat.ENCODING_PCM_16BIT
    private var bufferSize = 1024

    private var audioRecord: AudioRecord? = null
    private var recordingThread: Thread? = null
    private var recordingFlag = false

    private var file: File? = null
    private val lock = ReentrantLock()
    private var mLame: AndroidLame? = null

    constructor() {
    }

    public fun setChannelConfig(channelConfig: Int) {
        this.recorderChannelConfig = channelConfig
    }

    public fun setAudioFormat(audioFormat: Int) {
        this.recorderAudioFormat = audioFormat
    }

    public fun prepare(file: File) {
        lock.lock()
        if (this.file == null && audioRecord == null) {
            this.file = file
            var parentFile = file.parentFile
            if (!parentFile.exists() || !parentFile.isDirectory)
                parentFile.mkdirs()
            initBufferSizeAndSampleRate()

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                audioRecord = AudioRecord.Builder()
                    .setAudioSource(MediaRecorder.AudioSource.VOICE_COMMUNICATION)
                    .setAudioFormat(
                        AudioFormat.Builder()
                            .setSampleRate(sampleRate)
                            .setChannelMask(recorderChannelConfig)
                            .setEncoding(recorderAudioFormat)
                            .build()
                    )
                    .setBufferSizeInBytes(2 * bufferSize)
                    .build()
            } else {
                audioRecord = AudioRecord(
                    MediaRecorder.AudioSource.VOICE_COMMUNICATION,
                    sampleRate, recorderChannelConfig, recorderAudioFormat,2 * bufferSize
                )
            }

            var builder = LameBuilder()
                .setInSampleRate(audioRecord!!.sampleRate)
                .setOutChannels(audioRecord!!.channelCount)
//                .setOutSampleRate(audioRecord!!.sampleRate)
            mLame = AndroidLame(builder)
        }
        lock.unlock()
    }

    public fun startRecording() {
        lock.lock()
        if (file != null && !isRecording()) {
            audioRecord?.startRecording()
            recordingFlag = true

            recordingThread = Thread( { writeAudioDataToFile() }, "AudioRecorder Thread")
            recordingThread?.start()
        }
        lock.unlock()
    }

    public fun stopRecording() {
        lock.lock()
        if (isRecording()) {
            audioRecord?.stop()
            recordingFlag = false;
        }
        else {
            release()
        }
        lock.unlock()
    }

    public fun cancelRecording() {
        lock.lock()
        var filename = file?.absolutePath
        if (isRecording()) {
            audioRecord?.stop()
            recordingFlag = false;
        }
        else {
            release()
        }
        if (filename != null) {
            File(filename).delete()
        }
        lock.unlock()
    }

    public fun isRecording(): Boolean {
        return recordingFlag
    }

    private fun initBufferSizeAndSampleRate() {
        for (sampleRate in intArrayOf(44100, 22050, 11025, 16000, 8000)) {
            val bufferSize = AudioRecord.getMinBufferSize(sampleRate, recorderChannelConfig, recorderAudioFormat)
            if (bufferSize > 0) {
                this.sampleRate = sampleRate
                this.bufferSize = bufferSize
                break
            }
        }
    }

    private fun release() {
        audioRecord?.release()
        audioRecord = null
        recordingThread = null
        file = null

        mLame?.close()
        mLame = null
    }

    private fun writeAudioDataToFile() {
        val capacity = audioRecord!!.sampleRate * audioRecord!!.channelCount
        val recordBuffer = ShortArray(capacity)
        val lameBuffer = ByteArray(capacity)

        var fileOutputStream: FileOutputStream? = null

        try {
            fileOutputStream = FileOutputStream(file)
        } catch (e: FileNotFoundException) {
            e.printStackTrace()
        }

        if (fileOutputStream != null) {
            while (isRecording()) {
                val recorderSample = audioRecord!!.read(recordBuffer, 0, capacity)
                if (recorderSample > 0) {
                    val lameSample = mLame!!.encode(recordBuffer, recordBuffer, recorderSample, lameBuffer)

                    if (lameSample > 0) {
                        try {
                            fileOutputStream.write(lameBuffer, 0, lameSample)
                            fileOutputStream.flush()
                        } catch (e: IOException) {
                            e.printStackTrace()
                        }
                    }
                }
            }

            val flushSample = mLame!!.flush(lameBuffer)
            if (flushSample > 0) {
                try {
                    fileOutputStream.write(lameBuffer, 0, flushSample)
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }

            try {
                fileOutputStream.flush()
            } catch (e: IOException) {
                e.printStackTrace()
            }

            try {
                fileOutputStream.close()
                fileOutputStream = null
            } catch (e: IOException) {
                e.printStackTrace()
            }
            release()
        }
    }
}